// diamond.cpp
// by David Lambert 2013-01-17

#include <iostream>
#include <iomanip>
#include <cmath>
#include "diamond.h"

using namespace std;

Diamond::Diamond(const int initSize, const char initBorder, 
    const char initFill)
{
   // make sure size is > 1 and < 39. if not, set defaults
   if (initSize < 1)
      { size = 1; }
   else if (initSize > 39)
      { size = 39; }
   else
      { size = initSize; }

   // set border and fill chars, member functions do error-checking   
   SetBorder(initBorder);
   SetFill(initFill);
}

void Diamond::Grow()
{
   if (size < 39)
      { size++; }
}

void Diamond::Shrink()
{
   if (size > 1)
      { size--; }
}

void Diamond::SetBorder(char x)
{
   if (x < 33 || x > 126)   // limit to specific characters
      { border = 35; }
   else
      { border = x; }
}

void Diamond::SetFill(char x)
{
   if (x < 33 || x > 126)   // limit to specific characters
      { fill = 42; }
   else
      { fill = x; }
}

void Diamond::Draw() const
{
   // draw top half of diamond
   for (int line = 1; line <= size; line++) // traverse thru lines
   {
      for (int spc = 1; spc <= (size - line); spc++) // spaces for alignment
         { cout << " "; }
      for (int col = 1; col <= line; col++) // go thru columns in current line
      {
         if (col == 1 || col == line)   // if a border, show border character
            { cout << border << " "; }
         else              // if not a border, show fill character
            { cout << fill << " "; }
      }
      cout << endl;
   }
   
   // draw bottom half of diamond (sans top line)
   for (int line = (size - 1); line >= 1; line--) // traverse thru lines
   {
      for (int spc = 1; spc <= (size - line); spc++) // spaces for alignment
         { cout << " "; }
      for (int col = 1; col <= line; col++) // go thru columns in current line
      {
         if (col == 1 || col == line)   // if a border, show border character
            { cout << border << " "; }
         else              // if not a border, show fill character
            { cout << fill << " "; }
      }
      cout << endl;
   }
   cout << endl;
}

void Diamond::Summary() const
{   
   cout << setiosflags( ios::fixed | ios::showpoint )	<< setprecision( 2 );
   cout << "Size of diamond's side = " << GetSize() << " units.\n";
   cout << "Perimeter of diamond's side = " << Perimeter() << " units.\n";
   cout << "Area of diamond = " << Area() << " units.\n";
   cout << "Diamond looks like:\n";
   Draw();
}

int Diamond::GetSize() const
{
   return size;
}

int Diamond::Perimeter() const
{
   return size * 4;
}

double Diamond::Area() const
{
   return ((size * size)*(sqrt(3)/4))*2;
}
